﻿// [Book name], ISBN: [isbn]
// Kenn Scribner and Scott Seely
// This code supplied without any warranties, express or implied. 
// Code is free to use, public domain. 
// Have a nice day!

using System;
using System.Security.Cryptography;
using System.Security.Cryptography.X509Certificates;
using System.Text;
using System.Net;

namespace PhotoShared
{
    public enum ServiceType
    {
        Manager,
        Image
    }

    public static class Utility
    {
        public static string BaseUri{get;set;}

        static CredentialCache _credentialCache = null;
        public static CredentialCache CredentialCache
        {
            get
            {
                if (_credentialCache == null)
                {
                    _credentialCache = new CredentialCache();
                }
                return _credentialCache;
            }
        }

        public static NetworkCredential NetworkCredential
        {
            get;
            set;
        }

        public static WebClient CreateClient()
        {
            WebClient retval = new WebClient();
            retval.Credentials = CredentialCache;
            return retval;
        }

        public static WebRequest CreateRequest(Uri uri)
        {
            WebRequest retval = WebRequest.Create(uri);
            retval.Credentials = CredentialCache;
            retval.PreAuthenticate = true;
            return retval;
        }

        public static Uri CreateUri(ServiceType serviceType, string partialUri)
        {
            string svcName = null;
            switch (serviceType)
            {
                case ServiceType.Image:
                    svcName = "ImageManager.svc";
                    break;
                case ServiceType.Manager:
                    svcName = "UserManager.svc";
                    break;
            }
            Uri retval = new Uri(string.Format("{0}{1}/{2}",
                BaseUri,
                svcName,
                partialUri));
            return retval;
        }

        public static void ClearCredentialCache()
        {
            _credentialCache = null;
        }
    }
}
