﻿// [Book name], ISBN: [isbn]
// Kenn Scribner and Scott Seely
// This code supplied without any warranties, express or implied. 
// Code is free to use, public domain. 
// Have a nice day!

using System;
using System.Diagnostics;
using System.Linq;
using System.Net;
using System.Web;
using System.Windows.Forms;
using System.Xml;
using System.Xml.Linq;
using PhotoShared;
using System.Text;
using System.IO;
using System.Runtime.Serialization;

namespace WinFormPhotoClient
{
    public partial class LoginControl : UserControl
    {
        public event EventHandler LoggedIn;

        public LoginControl()
        {
            InitializeComponent();
        }

        public string LoggedInUserId { get { return txtUserName.Text; } }

        private void LoginControl_Load(object sender, EventArgs e)
        {

        }

        private void btnCreateAccount_Click(object sender, EventArgs e)
        {
            // Construct the call.
            Uri callUri = Utility.CreateUri(ServiceType.Manager,
              string.Format("CreateUser/{0}?email={1}",
              HttpUtility.UrlEncode(txtUserName.Text),
              HttpUtility.UrlEncode(txtEmailAddress.Text)));

            // Create the WebRequest
            WebRequest request = WebRequest.Create(callUri);
            request.Method = "PUT";
            Stream stream = request.GetRequestStream();
            request.ContentType = "application/xml";
            DataContractSerializer stringSer = new DataContractSerializer(typeof(string));
            stringSer.WriteObject(stream, txtPassword.Text);
            stream.Close();
            // Asynchronously wait for the response.
            request.BeginGetResponse(new AsyncCallback(CreateAccountResponse), request);

            // Disable the UI while waiting.
            this.Enabled = false;
        }

        void CreateAccountResponse(IAsyncResult result)
        {
            // Capture the request from the state.
            WebRequest request = (WebRequest)result.AsyncState;
            try
            {
                // Ask for the response. Any exceptions will get thrown here.
                WebResponse response = request.EndGetResponse(result);
                this.Invoke((MethodInvoker)delegate
                {
                    // Lack of exceptions means that we got a 200 OK back. 
                    if (DialogResult.Yes == MessageBox.Show(this, "Account created. Do you want to log in?",
                        "Success", MessageBoxButtons.YesNo, MessageBoxIcon.Information))
                    {
                        btnLogin_Click(null, EventArgs.Empty);
                    }
                });
            }
            catch (WebException we)
            {
                // Let the user know what happened.
                HttpWebResponse httpResponse = (HttpWebResponse)we.Response;
                if (httpResponse.StatusCode == HttpStatusCode.Conflict)
                {
                    // Conflict means the name was in use.
                    this.Invoke((MethodInvoker)delegate
                    {
                        MessageBox.Show(this, "This username is already in use. Can not create a new account.",
                            "Username in use", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                    });
                }
                else
                {
                    // Something else happened. 
                    this.Invoke((MethodInvoker)delegate
                    {
                        MessageBox.Show(this, string.Format("Code: {0}\r\nDescription: {1}.",
                            httpResponse.StatusCode.ToString(), httpResponse.StatusDescription),
                            "Failed to create a new user.", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                    });
                }
                Debug.WriteLine(we.ToString());
            }
            this.Invoke((MethodInvoker)delegate
            {
                // Re-enable the UI.
                this.Enabled = true;
            });
        }

        private void btnLogin_Click(object sender, EventArgs e)
        {
            Utility.ClearCredentialCache();
            // Clear any stored credentials and store the new ones.
            NetworkCredential credential = new NetworkCredential(txtUserName.Text,
                    txtPassword.Text);
            Utility.CredentialCache.Add(new Uri(Properties.Settings.Default.PhotoService),
                "Basic", credential);
            if (this.LoggedIn != null)
            {
                // Raise the LoggedIn event 
                this.LoggedIn(this, EventArgs.Empty);
            }
        }
    }
}
