﻿// [Book name], ISBN: [isbn]
// Kenn Scribner and Scott Seely
// This code supplied without any warranties, express or implied. 
// Code is free to use, public domain. 
// Have a nice day!

using System;
using System.Diagnostics;
using System.Linq;
using System.Net;
using System.Web;
using System.Windows;
using System.Windows.Threading;
using System.Xml;
using System.Xml.Linq;
using PhotoShared;
using System.IO;
using System.Runtime.Serialization;

namespace WpfPhotoClient
{
    /// <summary>
    /// Interaction logic for Window1.xaml
    /// </summary>
    public partial class Window1 : Window
    {
        delegate void PlainMethod();

        public Window1()
        {
            InitializeComponent();
        }

        private void btnLogin_Click(object sender, RoutedEventArgs e)
        {
            Utility.ClearCredentialCache();
            // Clear any stored credentials and store the new ones.
            NetworkCredential credential = new NetworkCredential(txtUsername.Text,
                    txtPassword.Password);
            Utility.CredentialCache.Add(new Uri(Properties.Settings.Default.PhotoService),
                "Basic", credential);
            ImageView imageView = new ImageView();
            imageView.LoggedInUserId = txtUsername.Text;
            App.Current.MainWindow = imageView;
            imageView.Show();
            Close();
        }

        void CreateAccountResponse(IAsyncResult result)
        {
            WebRequest request = (WebRequest)result.AsyncState;
            try
            {
                WebResponse response = request.EndGetResponse(result);
                Dispatcher.Invoke((PlainMethod)delegate
                {
                    if (MessageBoxResult.Yes == MessageBox.Show(this, "Account created. Do you want to log in?",
                        "Success", MessageBoxButton.YesNo, MessageBoxImage.Information))
                    {
                        btnLogin_Click(null, null);
                    }
                });
            }
            catch (WebException we)
            {
                HttpWebResponse httpResponse = (HttpWebResponse)we.Response;
                if (httpResponse.StatusCode == HttpStatusCode.Conflict)
                {
                    this.Dispatcher.Invoke((PlainMethod)delegate
                    {
                        MessageBox.Show(this, "This username is already in use. Can not create a new account.",
                            "Username in use", MessageBoxButton.OK, MessageBoxImage.Exclamation);
                        this.IsEnabled = true;
                    });
                }
                Debug.WriteLine(we.ToString());
            }
        }
        
        private void btnCreateAccount_Click(object sender, RoutedEventArgs e)
        {
            // Construct the call.
            Uri callUri = Utility.CreateUri(ServiceType.Manager,
              string.Format("CreateUser/{0}?email={1}",
              HttpUtility.UrlEncode(txtUsername.Text),
              HttpUtility.UrlEncode(txtEmailAddress.Text)));

            // Create the WebRequest
            WebRequest request = WebRequest.Create(callUri);
            request.Method = "PUT";
            Stream stream = request.GetRequestStream();
            request.ContentType = "application/xml";
            DataContractSerializer stringSer = new DataContractSerializer(typeof(string));
            stringSer.WriteObject(stream, txtPassword.Password);
            stream.Close();
            // Asynchronously wait for the response.
            request.BeginGetResponse(new AsyncCallback(CreateAccountResponse), request);
        }
    }
}
