﻿var XHRCreator = {
    // List of functions to try when creating a XHR.
    // Order is important, these are ranked from preferred mechanism
    // to least preferred.
    _factories: [function() { return new XMLHttpRequest(); },
        function() { return new ActiveXObject("Msxml2.XMLHTTP"); },
        function() { return new ActiveXObject("Microsoft.XMLHTTP"); }
    ],
    
    // Used to track which function to use when creating a new XHR.
    _factory: null,
    
    // Creates an XHR referencing the 'right' logic.
    CreateRequest: function() {
        var retval = null;
        if (this._factory == null) {
            for (var i = 0; i < this._factories.length; ++i) {
                try {
                    retval = this._factories[i]();
                    if (retval != null) {
                        this._factory = this._factories[i];
                        break;
                    }
                } catch (e) {
                    continue;
                }
            }
        }

        if (retval == null) {
            retval = this._factory();
        }
        return retval;
    }
};

var PhotoWebXHR = {

    LookupImages: function(lstId, divId) {
        var userId = document.getElementById(lstId);
        var showDeleteBar = true;
        if (userId == undefined) {
            userId = "@self";
        }
        else {
            showDeleteBar = false;
            userId = userId.value;
        }
        var xhr = XHRCreator.CreateRequest();
        if (xhr != undefined) {
            xhr.onreadystatechange = function() {
                var xmlDoc = null;
                if (xhr.readyState == 4 && xhr.status == 200) {
                    if (xhr.responseXML == undefined) {
                        if (window.ActiveXObject) {
                            xmlDoc = new ActiveXObject("Microsoft.XMLDOM");
                            xmlDoc.loadXML(xhr.responseText);
                        }
                    }
                    else {
                        xmlDoc = xhr.responseXML;
                    }
                    if (xmlDoc == null) {
                        alert("XML not supported by this browser. Failed to get a useable response: " + xhr.responseText);
                        return;
                    }
                    var imageItems = xmlDoc.getElementsByTagName("ImageItem");
                    var imageUrl;
                    var imageCaption;
                    var htmlText = "";
                    for (var i = 0; i < imageItems.length; ++i) {
                        var imageNode = imageItems[i];
                        var captionNode = imageNode.getElementsByTagName("Caption");
                        var urlNode = imageNode.getElementsByTagName("ImageUrl");
                        var descriptionNode = imageNode.getElementsByTagName("Description");
                        var idNode = imageNode.getElementsByTagName("ImageId");
                        var publicNode = imageNode.getElementsByTagName("PublicImage");
                        if (captionNode.length == 1 && urlNode.length == 1) {
                            var imgSrc;
                            var caption;
                            var description;
                            var imageId;
                            var publicImage;
                            if (urlNode[0].textContent == undefined) {
                                imgSrc = urlNode[0].text;
                                caption = captionNode[0].text;
                                description = descriptionNode[0].text;
                                imageId = idNode[0].text;
                                publicImage = publicNode[0].text;
                            } else {
                                imgSrc = urlNode[0].textContent;
                                caption = captionNode[0].textContent;
                                description = descriptionNode[0].textContent;
                                imageId = idNode[0].textContent;
                                publicImage = publicNode[0].textContent;
                            }
                            var deleteBar = showDeleteBar ? "<div class='Delete' onclick=\"PhotoWebXHR.Delete('" + imageId +
                                "');\" >X</div>" : "";
                            htmlText += "<div class='imageItem' id='" + imageId + "'>" +
                                deleteBar + "<div onclick='PhotoWebXHR.ShowDialog(\"" +
                                imageId + "\");' ><img class='actualImage' src='" +
                                imgSrc + "' alt='" +
                                caption + "' /><input type='hidden' value='" + description + "'/>" +
                                "<input type='hidden' value='" + publicImage + "'/><p>" + caption +
                                "</p></div></div>";
                        }
                    }
                    var divElement = document.getElementById(divId);
                    divElement.innerHTML = htmlText;
                }
            };
            xhr.open("GET", "/PhotoWebCh4/ImageManager.svc/Images/" + userId);
            xhr.send(null);
        }
    },
    Delete: function(divId) {
        var xhr = XHRCreator.CreateRequest();
        if (xhr != undefined) {
            xhr.onreadystatechange = function() {
                if (xhr.readyState == 4 && xhr.status == 200) {
                }
            };
            xhr.open("DELETE", "/PhotoWebCh4/ImageManager.svc/Image/" + divId);
            xhr.send(null);
            var theDiv = document.getElementById(divId);
            if (theDiv != undefined) {
                theDiv.parentNode.removeChild(theDiv);
                theDiv.outerHTML = "";
            }
        }
    },
    ShowDialog: function(divId) {
        var dialogContainer = document.getElementById("dialogContainer");
        var theDiv = document.getElementById(divId);
        var img = theDiv.getElementsByTagName("img");
        var caption = img[0].alt;
        var description = theDiv.getElementsByTagName("input")[0].value;
        var publicImage = theDiv.getElementsByTagName("input")[1].value == "true" ? "CHECKED" : "";
        var myDialog = document.getElementById("dialog");
        if (myDialog != undefined) {
            myDialog.parentNode.removeChild(myDialog);
        }
        dialogContainer.innerHTML =
    "<div id=\"dialog\">" +
            "<input type='hidden' value='" + divId + "' id='txtId'/>" +
            "<p><span class=\"dialogLabel\">Caption</span><input type=\"text\" id=\"txtCaption\" value=\"" + caption + "\"/></p>" +
            "<p><span class=\"dialogLabel\">Description</span><input type=\"text\" id=\"txtDescription\" value='" + description + "' /></p>" +
            "<p>Public Image <input id=\"chkPublicImage\" type=\"checkbox\" " + publicImage + " />" +
            "<p><input type=\"button\" value=\"OK\" onclick=\"PhotoWebXHR.SaveData();\"/><input type=\"button\" onclick=\"$('#dialog').dialog('close')\" value=\"Cancel\" /> </p>" +
    "</div>";

        $("#dialog").dialog({
            modal: true,
            overlay: {
                opacity: 0.7,
                background: "black"
            }
        });
    },

    SaveData: function() {
        // Capture edits.
        var txtId = document.getElementById("txtId").value;
        var txtCaption = document.getElementById("txtCaption").value;
        var txtDescription = document.getElementById("txtDescription").value;
        var chkPublicImage = document.getElementById("chkPublicImage").checked;
        
        // Save the edits back to the page.
        var theDiv = document.getElementById(txtId);
        theDiv.getElementsByTagName("p")[0].innerHTML = txtCaption;
        var img = theDiv.getElementsByTagName("img");
        img[0].alt = txtCaption;
        theDiv.getElementsByTagName("input")[0].value = txtDescription;
        theDiv.getElementsByTagName("input")[1].value = chkPublicImage;
        
        // Create the XHR    
        var xhr = XHRCreator.CreateRequest();
        if (xhr != undefined) {
            xhr.onreadystatechange = function() {
                if (xhr.readyState == 4) {
                    $('#dialog').dialog('close');
                    if (xhr.status != 200) {
                        alert('Update failed. Returned status was: ' + xhr.status);
                    }
                }
            };
            xhr.open("PUT", "/PhotoWebCh4/ImageManager.svc/Image/" + txtId);
            xhr.setRequestHeader("Content-Type", "application/xml")
            var upload = "<ImageItem xmlns=\"http://www.scottseely.com/RESTBook/2008\">" +
                "<Caption>" + txtCaption + "</Caption>" +
                "<Description>" + txtDescription + "</Description>" +
                "<PublicImage>" + chkPublicImage + "</PublicImage>" +
                "</ImageItem>";
            xhr.send(upload);
        }
    }

};
