﻿// [Book name], ISBN: [isbn]
// Kenn Scribner and Scott Seely
// This code supplied without any warranties, express or implied. 
// Code is free to use, public domain. 
// Have a nice day!

using System;
using System.Security.Cryptography;
using System.Security.Cryptography.X509Certificates;
using System.Text;
using PhotoShared;

namespace PhotoWeb
{
    static class Utility
    {
        const string CertificateSubject = "CN=PhotoWeb";

        public static string EncryptBase64(string value)
        {
            string retval = null;
            X509Store store = new X509Store(StoreLocation.LocalMachine);
            store.Open(OpenFlags.ReadOnly);

            X509Certificate2Collection coll = store.Certificates.Find(X509FindType.FindBySubjectDistinguishedName, CertificateSubject, false);
            if (coll.Count == 1)
            {
                X509Certificate2 cert2 = coll[0];
                RSACryptoServiceProvider rsa = (RSACryptoServiceProvider)cert2.PrivateKey;
                retval = Convert.ToBase64String(rsa.Encrypt(Encoding.UTF8.GetBytes(value), true));
            }
            return retval;
        }

        public static string DecryptBase64(string value)
        {
            string retval = null;
            X509Store store = new X509Store(StoreLocation.LocalMachine);
            store.Open(OpenFlags.ReadOnly);

            X509Certificate2Collection coll = store.Certificates.Find(X509FindType.FindBySubjectDistinguishedName, CertificateSubject, false);
            if (coll.Count == 1)
            {
                X509Certificate2 cert2 = coll[0];
                RSACryptoServiceProvider rsa = (RSACryptoServiceProvider)cert2.PrivateKey;
                retval = Encoding.UTF8.GetString(rsa.Decrypt(Convert.FromBase64String(value), true));
            }
            return retval;
        }

        public static string Decrypt(byte[] value)
        {
            string retval = null;
            X509Store store = new X509Store(StoreLocation.LocalMachine);
            store.Open(OpenFlags.ReadOnly);

            X509Certificate2Collection coll = store.Certificates.Find(X509FindType.FindBySubjectDistinguishedName, CertificateSubject, false);
            if (coll.Count == 1)
            {
                X509Certificate2 cert2 = coll[0];
                RSACryptoServiceProvider rsa = (RSACryptoServiceProvider)cert2.PrivateKey;
                retval = Encoding.UTF8.GetString(rsa.Decrypt(value, true));
            }
            return retval;
        }

        public static bool TryParse(string value, out Guid id)
        {
            bool retval = true;
            try
            {
                id = new Guid(value);
            }
            catch
            {
                id = Guid.Empty;
                retval = false;
            }
            return retval;
        }

        public static ImageItem ConvertImage(Image image)
        {
            return new ImageItem()
            {
                ImageId = image.ImageId,
                Description = image.Description,
                Caption = image.Caption,
                PublicImage = image.PublicImage,
                UserName = image.UserName
            };
        }

    }
}
