﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using System.Security.Principal;
using System.Web;
using System.Web.Mvc;
using System.Web.Security;
using System.Web.UI;
using System.IO;
using System.Text;
using System.Net;

namespace AspNetMvcRestService.Models
{
    public class CodeXRCAuxServices
    {
        #region Error rendering methods

        public void RenderErrorNotAcceptableGet(ControllerContext context)
        {
            context.HttpContext.Response.StatusCode = (Int32)HttpStatusCode.NotAcceptable;
            context.HttpContext.Response.Headers.Add("Allow", "text/html,text/xml,application/json");
            context.HttpContext.Response.ContentType = "text/html";
            using (StreamWriter wtr = new StreamWriter((Stream)context.HttpContext.Items["OutputStream"]))
            {
                StringBuilder sb = new StringBuilder();
                sb.Append("<html><head><title>CodeXRC</title></head><body>");
                sb.Append("The accepted types you requested are not supported by this service.<br/>");
                sb.Append("Please request */* (returns HTML), text/html, text/xml, or application/json.");
                sb.Append("</body><html>");
                wtr.WriteLine(sb.ToString());
                wtr.Flush();
                wtr.Close();
            }
        }

        public void RenderErrorNotAcceptablePut(ControllerContext context)
        {
            context.HttpContext.Response.StatusCode = (Int32)HttpStatusCode.NotAcceptable;
            context.HttpContext.Response.ContentType = "text/html";
            using (StreamWriter wtr = new StreamWriter((Stream)context.HttpContext.Items["OutputStream"]))
            {
                StringBuilder sb = new StringBuilder();
                sb.Append("<html><head><title>CodeXRC</title></head><body>");
                sb.Append("The only acceptable content type for creating/updating a resource is 'text/xml'.<br/>");
                sb.Append("</body><html>");
                wtr.WriteLine(sb.ToString());
                wtr.Flush();
                wtr.Close();
            }
        }

        public void RenderErrorInternalError(ControllerContext context, string message)
        {
            context.HttpContext.Response.StatusCode = (Int32)HttpStatusCode.InternalServerError;
            context.HttpContext.Response.ContentType = "text/html";
            using (StreamWriter wtr = new StreamWriter((Stream)context.HttpContext.Items["OutputStream"]))
            {
                StringBuilder sb = new StringBuilder();
                sb.Append("<html><head><title>CodeXRC</title></head><body>");
                sb.Append("The service sustained an internal error and cannot process your request.<br/>");
                sb.AppendFormat("Error:<br/>{0}", message);
                sb.Append("</body><html>");
                wtr.WriteLine(sb.ToString());
                wtr.Flush();
                wtr.Close();
            }
        }

        public void RenderErrorNotFound(ControllerContext context)
        {
            context.HttpContext.Response.StatusCode = (Int32)HttpStatusCode.NotFound;
            context.HttpContext.Response.ContentType = "text/html";
            using (StreamWriter wtr = new StreamWriter((Stream)context.HttpContext.Items["OutputStream"]))
            {
                StringBuilder sb = new StringBuilder();
                sb.Append("<html><head><title>CodeXRC</title></head><body>");
                sb.Append("The resource you requested cannot be located.<br/>");
                sb.Append("</body><html>");
                wtr.WriteLine(sb.ToString());
                wtr.Flush();
                wtr.Close();
            }
        }

        public void RenderErrorForbidden(ControllerContext context)
        {
            context.HttpContext.Response.StatusCode = (Int32)HttpStatusCode.Forbidden;
            context.HttpContext.Response.ContentType = "text/html";
            using (StreamWriter wtr = new StreamWriter((Stream)context.HttpContext.Items["OutputStream"]))
            {
                StringBuilder sb = new StringBuilder();
                sb.Append("<html><head><title>CodeXRC</title></head><body>");
                sb.Append("You are not authorized to access the resource you requested.<br/>");
                sb.Append("</body><html>");
                wtr.WriteLine(sb.ToString());
                wtr.Flush();
                wtr.Close();
            }
        }

        public void RenderSuccessNoContent(ControllerContext context)
        {
            context.HttpContext.Response.StatusCode = (Int32)HttpStatusCode.NoContent;
        }

        public void RenderSuccessCreated(ControllerContext context)
        {
            context.HttpContext.Response.StatusCode = (Int32)HttpStatusCode.Created;
        }

        public void RenderSuccessOK(ControllerContext context)
        {
            context.HttpContext.Response.StatusCode = (Int32)HttpStatusCode.OK;
        }

        #endregion

        #region Content assignment methods

        public string RenderBinaryContent(ControllerContext context, FileDTO file)
        {
            // Delegate
            return RenderBinaryContent(context, file.LocalStorage + file.DisplayName);
        }

        public string RenderBinaryContent(ControllerContext context, string filePath)
        {
            // We'll need to open the file and convert the contents to Base64
            // before assigning the contents to the DTO.
            string contents = String.Empty;
            if (System.IO.File.Exists(filePath))
            {
                // Open the file stream for conversion
                using (FileStream fstm = new FileStream(filePath, FileMode.Open, FileAccess.Read, FileShare.Read))
                {
                    // Create a temporary stream to hold the contents
                    MemoryStream temp = new MemoryStream();

                    // Copy the contents of the file into the temporary stream
                    CopyStream(fstm, temp);

                    // Convert the contents and close the temporary stream
                    contents = Convert.ToBase64String(temp.ToArray());
                    temp.Close();
                }
            }

            return contents;
        }

        #endregion

        #region CopyStream

        public void CopyStream(Stream input, Stream output)
        {
            byte[] buffer = new byte[0x1000];
            Int32 bytesRead = 0;
            while ((bytesRead = input.Read(buffer, 0, buffer.Length)) > 0)
                output.Write(buffer, 0, bytesRead);
        }

        #endregion
    }
}
