﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using System.Web.Security;

namespace AspNetMvcRestService.Models
{
    public partial class CodeXRCDeleteService : CodeXRCServiceBase
    {
        public CodeXRCDeleteService(string[] directives) :
            base(directives)
        {
        }

        public override void ExecuteResult(ControllerContext context)
        {
            // Test the user's credentials
            if (context.HttpContext.User.IsInRole("Delete"))
            {
                try
                {
                    // Delete the indicated resource
                    DeleteResource(context);

                    // Done... Since we're not sending back anything in
                    // the entity, send back the NoContent return code.
                    _aux.RenderSuccessNoContent(context);
                }
                catch (Exception ex)
                {
                    // Couldn't accept any requested type
                    _aux.RenderErrorInternalError(context, ex.ToString());
                }
            }
            else
            {
                // User forbidden
                _aux.RenderErrorForbidden(context);
            }
        }

        private void DeleteResource(ControllerContext context)
        {
            // Pull the user's ID. This works here since we're using the SQL-based
            // membership provider.
            MembershipUser user = Membership.GetUser(context.HttpContext.User.Identity.Name);
            Guid ownerID = (Guid)user.ProviderUserKey;

            switch (_directives.Count())
            {
                case 0:
                default:
                    // Delete all projects for user(all projects, child folders, and files):
                    // http://{host}/{vdir}/CodeXRC
                    _dal.DeleteProjectsForUser(context, ownerID);
                    break;

                case 1:
                    // Delete specific project for user (all child folders and files):
                    // http://{host}/{vdir}/CodeXRC/{project}
                    _dal.DeleteProject(context, ownerID, _directives[0]);
                    break;

                case 2:
                    // Delete specific folder (all child folders and files):
                    // http://{host}/{vdir}/CodeXRC/{project}/{folder}
                    _dal.DeleteFolder(context, ownerID, _directives[0], _directives[1]);
                    break;

                case 3:
                    // Delete files by extension in specified folder:
                    // http://{host}/{vdir}/CodeXRC/{project}/{folder}/{file extension}
                    _dal.DeleteFileByType(context, ownerID, _directives[0], _directives[1], _directives[2]);
                    break;

                case 4:
                    // Delete specific file:
                    // http://{host}/{vdir}/CodeXRC/{project}/{folder}/{file extension}/{file name}
                    _dal.DeleteFile(context, ownerID, _directives[0], _directives[1], _directives[3] + "." + _directives[2]);
                    break;
            }
        }
    }
}
