﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using System.Security.Principal;
using System.Web;
using System.Web.Mvc;
using System.Web.Security;
using System.Web.UI;
using System.IO;
using System.Text;

namespace AspNetMvcRestService.Models
{
    public partial class CodeXRCGetService : CodeXRCServiceBase
    {
        private Int64 _contentLength = 0;

        public CodeXRCGetService(string[] directives) :
            base(directives)
        {
        }

        public override void ExecuteResult(ControllerContext context)
        {
            // Test the user's credentials
            if (context.HttpContext.User.IsInRole("View"))
            {
                try
                {
                    if (context.HttpContext.Request.AcceptTypes.Count() > 0)
                    {
                        // Loop through the collection of accepted types. The first
                        // one we hit we understand, take it...
                        foreach (string type in context.HttpContext.Request.AcceptTypes)
                        {
                            switch (type.ToLower())
                            {
                                case "text/html":
                                case "*/*":
                                    RenderHtml(context);
                                    return;

                                case "text/xml":
                                    RenderXml(context);
                                    return;

                                case "application/json":
                                    RenderJson(context);
                                    return;

                                default:
                                    // Try next one...
                                    break;
                            }
                        }

                        // Couldn't accept any requested type
                        _aux.RenderErrorNotAcceptableGet(context);
                    }
                    else
                    {
                        // Couldn't accept any requested type
                        _aux.RenderErrorNotAcceptableGet(context);
                    }
                }
                catch (Exception ex)
                {
                    // Couldn't accept any requested type
                    _aux.RenderErrorInternalError(context, ex.ToString());
                }
            }
            else
            {
                // User forbidden
                _aux.RenderErrorForbidden(context);
            }
        }

        protected Int64 GetContentLength(ControllerContext context)
        {
            // Check to see if this is a HEAD request. The output stream
            // for responses doesn't allow length query.
            Stream stream = context.HttpContext.Items["OutputStream"] as Stream;
            Int64 length = 0;
            if (stream != null && stream.CanSeek)
            {
                length = stream.Length;
            }

            return length;
        }
    }
}
