﻿using System;
using System.Security;
using System.Security.Principal;
using System.Web;
using System.Web.Security;

namespace AspNetMvcRestService.Security
{
    // We could use the GenericPrincipal and GenericIdentity, but I
    // wanted to show how we might implement them, and I wanted to
    // force the AuthenticationType in the IIdentity value to be
    // "Basic".
    [Serializable]
    public sealed class ServiceUser : IPrincipal
    {
        IIdentity _identity = null;
        string[] _roles = new string[0];

        internal ServiceUser(string username, bool isAuthenticated)
        {
            // Store the identity
            _identity = new ServiceIdentity(username, isAuthenticated) as IIdentity;

            // Get the roles for the user
            if (isAuthenticated)
            {
                _roles = Roles.GetRolesForUser(username);
            }
        }

        public bool IsInRole(string role)
        {
            // Look up role
            foreach (string testRole in _roles)
            {
                // Compare
                if (role.ToLowerInvariant() == testRole.ToLowerInvariant())
                {
                    // Found it...
                    return true;
                }
            }

            // Role not found
            return false;
        }

        public IIdentity Identity
        {
            get { return _identity; }
        }
    }

    [Serializable]
    public sealed class ServiceIdentity : IIdentity
    {
        private const string AuthenticationTypeString = "Basic";
        private string _username = String.Empty;
        private bool _isAuthenticated = false;
        private Guid _userID = Guid.Empty;

        internal ServiceIdentity(string username, bool isAuthenticated)
        {
            _username = username;
            _isAuthenticated = isAuthenticated;
        }

        public string AuthenticationType
        {
            get { return AuthenticationTypeString; }
        }

        public bool IsAuthenticated
        {
            get { return _isAuthenticated; }
        }

        public string Name
        {
            get { return _username; }
        }
    }
}
