﻿using System;
using System.Diagnostics;
using System.Runtime.Serialization;
using System.Web;
using System.Collections.Generic;

/// <remarks>
/// I derived this class as I would any class coming from a RESTful
/// service that I knew would be updated via XML. I took a copy of
/// the XML from the service and ran it through xsd.exe to create
/// a schema. With the schema in hand, I ran xsd.exe again to create
/// a C# file.
/// 
/// Then it got interesting. The C# file you get is tailored for
/// the XML Serializer, but here I wanted to use the runtime XML
/// serialializer to make sure the XML would match. I found I also
/// then had to flex the server side a bit to make some of the
/// properties optional (or marked as optional...they always were
/// intended to be issued to the client, not client to service). 
/// </remarks>
[DataContract(Namespace = "http://www.endurasoft.com/REST.NET/", Name = "Project")]
public partial class LocalProjectDTO
{

    private string projectIDField;

    private string displayNameField;

    private string linkField;

    // Added to make insertion into the arrays
    // easier...we won't need to know the number
    // of elements beforehand, making the
    // recursive calls to insert files and
    // folders easier. This won't serialize...
    private List<LocalFolderDTO> foldersListField = new List<LocalFolderDTO>();

    /// <remarks/>
    [DataMember]
    public string ProjectID
    {
        get
        {
            return this.projectIDField;
        }
        set
        {
            this.projectIDField = value;
        }
    }

    /// <remarks/>
    [DataMember]
    public string DisplayName
    {
        get
        {
            return this.displayNameField;
        }
        set
        {
            this.displayNameField = value;
        }
    }

    /// <remarks/>
    [DataMember]
    public string Link
    {
        get
        {
            return this.linkField;
        }
        set
        {
            this.linkField = value;
        }
    }

    /// <remarks/>
    [DataMember]
    public LocalFolderDTO[] Folders
    {
        get
        {
            return FoldersList.ToArray();
        }
        set
        {
            FoldersList.AddRange(value);
        }
    }

    // This won't be serilized...
    public List<LocalFolderDTO> FoldersList
    {
        get { return foldersListField; }
    }
}

