﻿// Effective REST Services via .NET, ISBN: 0321613406
// Kenn Scribner and Scott Seely
// This code supplied without any warranties, express or implied. 
// Code is free to use, public domain. 
// Have a nice day!
using System;
using System.Net;
using System.Text;
using System.Web;
using System.Web.Security;

namespace PhotoWeb
{
    public class AuthModule : IHttpModule
    {
        /// <summary>
        /// You will need to configure this module in the web.config file of your
        /// web and register it with IIS before being able to use it. For more information
        /// see the following link: http://go.microsoft.com/?linkid=8101007
        /// </summary>
        #region IHttpModule Members

        public void Dispose()
        {
            //clean-up code here.
        }

        public void Init(HttpApplication context)
        {
            // Hook the authentication request event
            context.AuthenticateRequest += new EventHandler(OnAuthenticateRequest);
        }

        #endregion

        #region Event handlers

        void OnAuthenticateRequest(object sender, EventArgs e)
        {
            // Pull the current context
            HttpContext context = HttpContext.Current;
            bool isAuthorized = false;
            string rawUrl = context.Request.RawUrl.ToUpper();
            if (!rawUrl.Contains("USERMANAGER.SVC") &&
                !rawUrl.Contains("IMAGEHANDLER.ASHX") &&
                !rawUrl.EndsWith(".RSS") &&
                !rawUrl.EndsWith(".ATOM") &&
                !rawUrl.EndsWith("FEEDIMAGE.JPG"))
            {
                // Look for the authorization header
                string authHeader = context.Request.Headers["Authorization"];
                if (!String.IsNullOrEmpty(authHeader) && authHeader.ToUpper().StartsWith("BASIC"))
                {
                    // Pull the credentials
                    string[] credentials = stripCredentials(authHeader);

                    // Authenticate
                    if (credentials.Length == 2 &&
                        Membership.ValidateUser(credentials[0], credentials[1]))
                    {
                        // Set the caller's credentials
                        context.User = new ServiceUser(credentials[0], true);

                        // Mark as authorized
                        isAuthorized = true;
                    }
                }
                if (!isAuthorized)
                {
                    // Assign the status code
                    context.Response.StatusCode = (Int32)HttpStatusCode.Unauthorized;

                    // Calculate the realm
                    string realm = String.Format("Basic realm=\"{0}\"", context.Request.Url.GetLeftPart(UriPartial.Path));

                    // Add the authentication header
                    context.Response.AddHeader("WWW-Authenticate", realm);

                    // End the request
                    context.Response.End();
                }
            }
        }

        #endregion

        #region Helpers

        string[] stripCredentials(string authHeader)
        {
            // Remove the "basic"
            string rawCredentials = authHeader.Substring(6).Trim();

            // Remove Base64 encoding
            byte[] bytes = Convert.FromBase64String(rawCredentials);
            string credentials = UTF8Encoding.UTF8.GetString(bytes);

            // Return separated string array
            return credentials.Split(':');
        }

        #endregion
    }
}
