﻿// Effective REST Services via .NET, ISBN: 0321613406
// Kenn Scribner and Scott Seely
// This code supplied without any warranties, express or implied. 
// Code is free to use, public domain. 
// Have a nice day!
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.ServiceModel.Activation;
using System.ServiceModel.Syndication;
using System.ServiceModel.Web;
using System.Web;

namespace PhotoWeb
{
    [AspNetCompatibilityRequirements(RequirementsMode = AspNetCompatibilityRequirementsMode.Required)]
    public class PhotoFeed : IPhotoFeed
    {
        public SyndicationFeedFormatter CreateFeed()
        {
            Uri requestUri = HttpContext.Current.Request.Url;

            // Look at the URL to figure out which user and the response format
            // (Atom | RSS).
            string fileName = requestUri.Segments[requestUri.Segments.Length - 1];
            string username = Path.GetFileNameWithoutExtension(fileName);
            string feedType = Path.GetExtension(fileName.ToUpper());
            SyndicationFeedFormatter formatter = null;
            SyndicationFeed feed = new SyndicationFeed(string.Format("Photo feed for user {0}",
                username), "A Photo Feed", HttpContext.Current.Request.Url);

            // Allow an image to show up for this feed once someone subscribes.
            feed.ImageUrl = new Uri(string.Format("http://{0}{1}/FeedImage.jpg",
                HttpContext.Current.Request.Url.Authority,
                HttpContext.Current.Request.ApplicationPath));

            // Pick the feed.
            if (feedType.Equals(".ATOM"))
            {
                formatter = feed.GetAtom10Formatter();
            }
            else if (feedType.Equals(".RSS"))
            {
                formatter = feed.GetRss20Formatter();
            }
            else
            {
                // We don't understand the request type, so 
                // tell the caller that the resource they want can't
                // be found.
                WebOperationContext.Current.OutgoingResponse.StatusCode =
                    System.Net.HttpStatusCode.NotFound;
                return null;
            }

            using (PhotoWebDbDataContext dataContext =
                new PhotoWebDbDataContext())
            {
                // Get all the public images for the user in question
                var images = from image in dataContext.Images
                             where
                                 image.UserName == username &&
                                 image.PublicImage
                             select image;

                // No public images means no resource to return.
                // Maybe the user doesn't exist?
                if (images.Count() == 0)
                {
                    WebOperationContext.Current.OutgoingResponse.StatusCode =
                        System.Net.HttpStatusCode.NotFound;
                    return null;
                }

                List<SyndicationItem> items = new List<SyndicationItem>();
                foreach (var image in images)
                {
                    // Create and add the SyndicationItem to the feed
                    SyndicationItem si = new SyndicationItem(image.Caption, image.Description,
                                 new Uri(string.Format("http://{0}{1}/ImageHandler.ashx/{2}",
                                     HttpContext.Current.Request.Url.Authority,
                                     HttpContext.Current.Request.ApplicationPath,
                                     image.ImageId.ToString("D"))), image.ImageId.ToString(),
                                     new DateTimeOffset(image.Updated.Value));
                    items.Add(si);
                }
                feed.Items = items;
            }
            return formatter;
        }
    }
}
