﻿// Effective REST Services via .NET, ISBN: 0321613406
// Kenn Scribner and Scott Seely
// This code supplied without any warranties, express or implied. 
// Code is free to use, public domain. 
// Have a nice day!
using System.Collections.Generic;
using System.Data.Services.Common;
using System.Linq;
using System.Web;
using PhotoShared;

namespace PhotoWeb
{
    [DataServiceKey("Username")]
    public class ImageUser
    {
        public string Username { get; set; }

        public List<ImageItem> Images
        {
            get
            {
                ImageManager imgMgr = new ImageManager();
                return new List<ImageItem>(
                    imgMgr.GetImagesForUser(this.Username));
            }
        }
    }

    public class UserDataService
    {
        public IQueryable<ImageUser> Users
        {
            get
            {
                using (PhotoWebDbDataContext dataContext = new PhotoWebDbDataContext())
                {
                    var users = (from Image image in dataContext.Images
                                 select new ImageUser() { Username = image.UserName }).
                            Distinct().OrderBy(n => n.Username);
                    List<ImageUser> retval = new List<ImageUser>(users);
                    return retval.AsQueryable<ImageUser>();
                }
            }
        }

        public IQueryable<ImageItem> Images
        {
            get
            {
                List<ImageItem> retval = new List<ImageItem>();
                IEnumerable<Image> images = null;
                using (PhotoWebDbDataContext dataContext = new PhotoWebDbDataContext())
                {
                    images = from image in dataContext.Images
                             where
                                 image.PublicImage
                             select image;
                    string baseUrl = string.Format(
                        "http://{0}{1}/ImageHandler.ashx",
                        HttpContext.Current.Request.Url.Authority,
                        HttpContext.Current.Request.ApplicationPath);
                    foreach (var image in images)
                    {
                        ImageItem imageItem = Utility.ConvertImage(image);
                        imageItem.ImageUrl = string.Format("{0}/{1}", baseUrl, image.ImageId.ToString("D"));
                        retval.Add(imageItem);
                    }
                }
                return retval.AsQueryable<ImageItem>();
            }
        }
    }
}
